#!/bin/sh

set -e

PRINTF=/usr/bin/printf

check_for_program()
{
    if ! which "$1" >/dev/null 2>&1; then
        echo "Error: $1 is not in your PATH"
        exit 1
    fi
}

usage()
{
    local script
    script=`basename $0`
    local padsss
    padsss=`echo "$script" | tr '_a-zA-Z0-9-' '[ *]'`

    echo "USAGE: $script {[-l|--extents ExtentCount] | [-L|--size Size]}" >&2
    echo "       $padsss -n|--name SnapshotLVName -N|--vgname SnapshotVGName" >&2
    echo "       $padsss [-s|--snapshot] OriginPath" >&2
}


# Pre-flight checks

check_for_program blkid
check_for_program lvchange
check_for_program lvcreate
check_for_program lvrename
check_for_program udevadm
check_for_program vgimportclone


# Argument processing

snapshot_lvname=
snapshot_vgname=
origin_lvpath=
passthrough_args=

while [ $# -gt 0 ]; do
    case "$1" in
    --)
        if [ -n "$origin_lvpath" -o $# -gt 2 ]; then
            echo "Too many arguments" >&2
            usage
            exit 1
        fi

        origin_lvpath="$2"
        break
        ;;

    -*)
        ;;

    *)
        if [ -n "$origin_lvpath" ]; then
            echo "Too many arguments" >&2
            usage
            exit 1
        fi

        origin_lvpath="$1"
        shift
        continue
    esac
            
    if [ $# -lt 2 ]; then
        echo "Missing value for '$1' argument" >&2
        usage
        exit 1
    fi

    case $1 in
    -l|--extents|-L|--size)
        passthrough_args="$passthrough_args `$PRINTF '%q %q' "$1" "$2"`"
        ;;

    -n|--name)
        snapshot_lvname="$2"
        ;;

    -N|--vgname)
        snapshot_vgname="$2"
        ;;

    -s|--snapshot)
        if [ -n "$origin_lvpath" ]; then
            echo "Too many arguments" >&2
            usage
            exit 1
        fi

        origin_lvpath="$2"
        ;;

    *)
        echo "Unknown argument '$1'" >&2
        usage
        exit 1
    esac

    shift; shift
done

# Check for required arguments

if [ -z "$origin_lvpath" ]; then
    echo "An origin LV path must be specified" >&2
    usage
    exit 1
fi

if [ -z "$snapshot_lvname" ]; then
    echo "The snapshot LV name must be specified" >&2
    usage
    exit 1
fi

if [ -z "$snapshot_vgname" ]; then
    echo "The snapshot VG name must be specified" >&2
    usage
    exit 1
fi

# Validate arguments

origin_lvpath="${origin_lvpath#/dev/}"
origin_lvname="${origin_lvpath#*/}"
pending_lvname="pending-snap-$origin_lvname"
pending_lvpath="${origin_lvpath%%/*}/$pending_lvname"
snapshot_lvpath="${origin_lvpath%%/*}/$snapshot_lvname"

if [ ! -e "/dev/$origin_lvpath" ]; then
    echo "The specified origin LV does not exist or is not active"
    exit 2
fi

origin_info=`blkid -o export "/dev/$origin_lvpath"`
eval "$origin_info"

if [ x"$TYPE" != xLVM2_member ]; then
    echo "The specified origin LV does not appear to contain LVM data"
    exit 2
fi

if [ -e "/dev/$snapshot_lvpath" ]; then
    echo "The specified snapshot LV already exists" >&2
    exit 2
fi


# Perform the four steps required to safely snapshot the OPC
# user's LVM pool

set -x

lvcreate $passthrough_args                      \
         --setactivationskip y                  \
         --name "$pending_lvname"               \
         --snapshot "$origin_lvpath"

vgimportclone --basevgname "$snapshot_vgname"   \
              "/dev/$pending_lvpath" ||
    (rc=$?; lvremove --force "$pending_lvpath"; exit $rc)

# XXX: Avoid race between processing the clone import triggers in udev and
# the following lvchange command.
udevadm settle --timeout=10 || true

lvchange --setactivationskip n                  \
         "$pending_lvpath" ||
    (rc=$?; lvremove --force "$pending_lvpath"; exit $rc)

lvrename "$pending_lvpath" "$snapshot_lvpath" ||
    (rc=$?; lvremove --force "$pending_lvpath"; exit $rc)
